makeStratDomainPlot <- function(ageDepthModel,
                                transformed_ts,
                                axis_limits = c(-1, 1),
                                trait_name = "",
                                plotSeaLevel = TRUE,
                                plot_hiatuses = TRUE) {
  #' @title plot evolution in stratigraphic column
  #'
  #' @description
  #' Plots trait evolution of lineages in the stratigraphic domain
  #'
  #' @param ageDepthModel list generated by the function getAgeDepthModel
  #' @param ts_list list generated by the
  #' function getts_list
  #' @param axis_limits numeric vector of length 2. Specifies y axis limits
  #' @param trait_name Character string. Y axis label
  #' @param plotSeaLevel Logical. Should sea level be shown?
  #' @param plot_hiatuses Logical. Should hiatuses in strat. column be shown?
  #'
  #'
  #' @return a plot
  #'

  #### Determine y axis limits ####
  ymin <- min(axis_limits)
  ymax <- max(axis_limits)
  if (ymin == ymax) { # if duplicate values are provided: catch error
    ymin <- -1
    ymax <- 1
  }

  #### Generate empty plot ####
  plot(
    NULL,
    ylim = range(ageDepthModel$heightRaw),
    xlim = c(ymin, ymax),
    ylab = "Height [m]",
    xlab = "",
    type = "l"
  )

  #### Add title ####
  title(
    main = "Trait Evolution Observed in\nthe Stratigraphic Column",
    line = 2
  )

  #### Add y axis label ####
  mtext(
    text = trait_name,
    side = 1,
    line = 2.5
  )

  #### Indicate x axis ####
  lines(
    y = range(ageDepthModel$heightRaw),
    x = c(0, 0),
    lwd = zero_axis_lwd,
    lty = zero_axis_lty,
    col = zero_axis_col
  )

  #### plot evolutionary  simulations ####
  for (i in seq_along(transformed_ts)) {
    # see Hohmann (2021) https://doi.org/10.2110/palo.2020.038
    # for theoretical background
    lines(
      x = transformed_ts[[i]]$val,
      y = transformed_ts[[i]]$pos,
      col = trait_cols[i],
      lwd = trait_lwds[i],
      lty = trait_ltys[i]
    )
  }

  #### plot stratigraphic position of hiatuses ####
  if (plot_hiatuses) {
    for (i in seq_along(ageDepthModel$hiatusHeights)) {
      lines(
        y = rep(ageDepthModel$hiatusHeights[i], 2),
        x = c(ymin, ymax),
        col = hiatus_strat_col,
        lwd = hiatus_strat_lwd,
        lty = hiatus_strat_lty
      )
    }
  }

  #### Plot Sea Level ####
  if (plotSeaLevel) {
    lines(
      y = ageDepthModel$heightRaw,
      x = seaLevel / max(seaLevel) * 0.5 * (ymax - ymin) + 0.5 * (ymax + ymin),
      col = sl_col,
      lwd = sl_lwd,
      lty = sl_lty
    )

    mtext(
      text = "Sea Level",
      side = 3,
      col = sl_col,
      line = 0
    )

    axis(
      side = 3,
      at = c(ymin, ymax),
      labels = c("Low", "High"),
      col.ticks = sl_col,
      col.axis = sl_col
    )
  }
}
