makeTimeDomainPlot <- function(ts_list, 
                               ageDepthModel, 
                               axis_limits = c(-1,1),
                               trait_name = "", 
                               plotSeaLevel = FALSE, 
                               plot_time_gaps = FALSE,
                               plot_strat_info = FALSE) {
  
  #' @title generate time domain plot
  #' 
  #' @description
  #' Generates time domain plots to visualize evolutionary simulations
  #' 
  #' @param ts_list output generated by 
  #' the function getts_list
  #' @param ageDepthModel output generated by the function getAgeDepthModel
  #' @param axis_limits numeric vector of length 2, limits of the y axis
  #' @param trait_name character vector, y axis label
  #' @param plotSeaLevel Logical. should the sea level be shown
  #' @param plot_time_gaps Logical, should gaps in the record be shown
  #' @param plot_strat_info Logical, should strat info 
  #' (sea level & gaps) be shown
  #' 
  #' @return A plot
  #'
  
  #### Define y axis limits ####
  ymin = min(axis_limits)
  ymax = max(axis_limits)
  if (ymin == ymax){ # exception: identical input values
    ymin = -1
    ymax = 1
  }

  #### Generate empty plot ####
  plot(
    NULL,
    xlim = range(time_myr),
    ylim = c(ymin, ymax),
    xlab = "Time [Ma]",
    ylab = "",
    type = "l",
    main = "Trait Evolution in Time Domain"
  )
  
  #### plot x axis as reference ####
  lines(
    x = range(time_myr),
    y = c(0, 0),
    col = zero_axis_col,
    lty = zero_axis_lty,
    lwd = zero_axis_lwd
  )
  
  #### plot gaps ####
  if (plot_time_gaps & plot_strat_info) {
    for (i in seq_along(ageDepthModel$hiatusTimeList)) {
      rect(
        xleft = ageDepthModel$hiatusTimeList[[i]]$hiatusStart,
        xright = ageDepthModel$hiatusTimeList[[i]]$hiatusEnd,
        ybottom = ymin,
        ytop = ymax,
        col = removed_time_col,
        lty = 0
      )
    }
  }
  
  #### add y axis label ####
  mtext(
    text = trait_name,
    side = 2,
    line = 2.5
  )
  
  #### plot evolutionary simulations ####
  for (i in seq_along(ts_list)) {
    lines(
      x = ts_list[[i]]$time,
      y = ts_list[[i]]$val,
      col = trait_cols[i],
      lwd = trait_lwds[i],
      lty = trait_ltys[i]
    )
  }
  
  #### plot sea level ####
  if (plotSeaLevel & plot_strat_info) {
    lines(
      x = time_myr, 
      y = seaLevel / max(seaLevel) * 0.5 * (ymax - ymin) + 0.5 * (ymax + ymin),
      col = sl_col,
      lwd = sl_lwd,
      lty = sl_lty
    )
    mtext(
      text = "Sea Level",
      side = 4,
      col = sl_col,
      line = 1
    )
    axis(
      side = 4,
      at = c(ymin, 0.5 * (ymax + ymin), ymax),
      labels = c("Low", "", "High"),
      col.ticks = sl_col,
      col.axis = sl_col
    )
  }
  
  
}
