prepare_download_strat_pal <- function(file,
                                       transformed_ts,
                                       dist_from_shore_km,
                                       sampling_strategy,
                                       no_of_samples,
                                       dist_between_samples,
                                       trait_name,
                                       mode,
                                       ...) {
  #' @title prepare_download_strat_pal
  #'
  #' @description generates csv file for download in strat pal tab
  #'
  #' @param file path of temp file
  #' @param transformed_ts output generated by function transfrom_ts
  #' @param dist_from_shore_km distance from shore in km
  #' @param sampling_strategy CHaracter string, either "Fixed Distance"
  #'  or "Fixed Number"
  #' @param no_of_samples positive integer. Number of samples taken
  #' along the strat column
  #' @param dist_between_samples positive scalar. Distance between
  #' samples in strat column (in meter)
  #' @param trait_name character string, name of trait
  #' @param mode character string, mode of evolution
  #' @param ... numeric. parameters for modes of evolution
  #'  Entries 1-3 specify random walk.
  #'  Entries 4-5 specify stasis
  #'  Entries 6-9 specify Ornstein-Uhlenbeck
  #'
  #' @return a csv file
  #'

  ## write strat. position and time
  df <- data.frame(strat_pos_m = transformed_ts[[1]]$pos, time_myr = transformed_ts[[1]]$time)
  # write trait values of all lineages
  for (i in seq_along(transformed_ts)) {
    df[[paste("lineage_", i, "_", trait_name, sep = "")]] <- transformed_ts[[i]]$val
  }

  ## add metadata to file
  metadata <- rep("", length(transformed_ts[[1]]$pos))


  # sampling strategy
  if (sampling_strategy == "Fixed Number") {
    df[["sampling_strategy"]] <- replace(metadata, 1, "Fixed Number")
    df[["number_of_samples"]] <- replace(metadata, 1, no_of_samples)
  }
  if (sampling_strategy == "Fixed Distance") {
    df[["sampling_strategy"]] <- replace(metadata, 1, "Fixed Distance")
    df[["distance_between_samples_m"]] <- replace(metadata, 1, dist_between_samples)
  }

  # distance from shore
  df[["distance_from_shore_km"]] <- replace(metadata, 1, dist_from_shore_km)

  # evo params & modes
  parameters <- list(...)
  if (mode == "Random Walk") {
    sigma <- parameters[[1]]
    mu <- parameters[[2]]
    x0 <- parameters[[3]]
    df[["mode_of_evolution"]] <- replace(metadata, 1, "Random Walk")

    df[["variability"]] <- replace(metadata, 1, sigma)
    df[["drift"]] <- replace(metadata, 1, mu)
    df[["initial_trait_value"]] <- replace(metadata, 1, x0)
  }
  if (mode == "Stasis") {
    mean <- parameters[[4]]
    variance <- parameters[[5]]

    df[["mode_of_evolution"]] <- replace(metadata, 1, "Stasis")

    df[["mean_trait_value"]] <- replace(metadata, 1, mean)
    df[["variance"]] <- replace(metadata, 1, variance)
  }

  if (mode == "Ornstein-Uhlenbeck") {
    mu <- parameters[[6]]
    theta <- parameters[[7]]
    sigma <- parameters[[8]]
    x0 <- parameters[[9]]

    df[["mode_of_evolution"]] <- replace(metadata, 1, "Ornstein-Uhlenbeck")

    df[["long_term_mean"]] <- replace(metadata, 1, mu)
    df[["pressure_of_selection"]] <- replace(metadata, 1, theta)
    df[["volatility"]] <- replace(metadata, 1, sigma)

    df[["initial_trait_value"]] <- replace(metadata, 1, x0)
  }

  # citation
  df[["citation"]] <- replace(metadata, 1, citation_text)

  ## generate file for dowload
  write.csv(df, file = file, row.names = TRUE)
}
